#include "ecl.h"


volatile uint8_t ecl_bytepos = 0;
volatile uint8_t ecl_pos = 0;
volatile uint8_t ecl_packet[ECL_PACKET_SIZE];
ecl_data_t ecl_data;

//I/Os und Register fr die Dekodierung initialisieren
inline void ecl_init() {
	ECL_DDR &= ~(1<<ECL_PIN); //INT0 als Input

	//Ich wnschte, es ginge schner...
#if ECL_TMR_PRESCALER == 1
	TCCR0 = TIMER0_PRESCALER_1;
	#define ECL_TMR_PRESCALER_SET
#endif
#if ECL_TMR_PRESCALER == 8
	TCCR0 = TIMER0_PRESCALER_8;
	#define ECL_TMR_PRESCALER_SET
#endif
#if ECL_TMR_PRESCALER == 64
	TCCR0 = TIMER0_PRESCALER_64;
	#define ECL_TMR_PRESCALER_SET
#endif
#if ECL_TMR_PRESCALER == 256
	TCCR0 = TIMER0_PRESCALER_256;
	#define ECL_TMR_PRESCALER_SET
#endif
#if ECL_TMR_PRESCALER == 1024
	TCCR0 = TIMER0_PRESCALER_1024;
	#define ECL_TMR_PRESCALER_SET
#endif
#ifndef ECL_TMR_PRESCALER_SET
	#error ECL_PRESCALER not properly set!
#endif

	TIMER0_IRQ_DISABLE();
	
	//INT0 bei entsprechender Flanke auslsenlassen und aktivieren
#if ECL_SIG_INVERTED == 1
	INT0_ON_RISE();
#else
	INT0_ON_FALL();
#endif
	INT0_IRQ_ENABLE();
}


//Dekodieren eines Pakets
uint8_t ecl_decode() {

	//Ist das Teilpaket schon vollstndig eingelesen?
	if(ecl_pos != 17) {
		return ECL_S_NOT_READY;
	}

	ecl_pos++;

	uint8_t buffpos = ecl_bytepos;

	uint8_t mypos = buffpos;
	//Das letztes empfangene Byte prfen - ist beim vollstndigen Empfang eines Pakets 0x0D
	if(mypos == 0) mypos = ECL_PACKET_SIZE;
	mypos--;
	if(ecl_packet[mypos] != 0x0D || ecl_packet[buffpos] != 0x00) {
		return ECL_S_NOT_COMPLETE;
	}

	/**
	 * Es scheint also ein Paket empfangen geworden zu sein
	 * Um das Paket besser verarbeiten zu knnen, werden die Eintrge nun geshiftet.
	 */

	uint8_t buff[8];
	uint8_t checksum = 0;

	for(uint8_t i = 0; i < 8; i++) {
		buffpos++;
		if(buffpos == ECL_PACKET_SIZE) { buffpos = 0; }
		buff[i] = ecl_packet[buffpos];
		checksum += ecl_packet[buffpos];
	}

#if ECL_DUMP_PACKET == 1
	//Empfangene Daten auf den UART legen
	for(uint8_t i = 0; i < 8; i++) {
		ECL_DUMP_HEX(buff[i]);
	}
	ECL_DUMP_CHR('\n');
#endif

	/** 
	 * "Prfsumme" vergleichen
	 * Die Entwickler von Danfoss haben das Wort Prfsumme wrtlich genommen.
	 * Sie besteht aus der Summe der einzelnen Bytes (ohne die letzten beiden Bytes)
	 * und haben dabei anscheinend sogar noch einen Fehler gemacht.
	 * Zumindest ist das MSB der Prfsumme bei manchen korrekten Paketen gekippt.
	 * Prinzipbedingt ist diese Art der Sicherung nicht wirklich wirkungsvoll, da 
	 * sich mehrfache Fehler sehr schnell auslschen...
	 **/
	buffpos++;
	if(buffpos == ECL_PACKET_SIZE) { buffpos = 0; }
	if(((checksum ^ ecl_packet[buffpos]) & 0x7F) != 0) {
		
		ECL_DEBUG_S("Checksum error - expected ");
		ECL_DEBUG_H(ecl_packet[buffpos]);
		ECL_DEBUG_S(" but received ");
		ECL_DEBUG_H(checksum);
		ECL_DEBUG_C('\n');

		return ECL_S_CHECKSUM_ERROR;
	}
	


	/**
	 * Datenpaket auswerten
	 * Dabei ist Quell-/Zieladresse zunchst unrelevant, da Pakettypen nicht doppelt verwendet werden.
	 **/

#if ECL_USE_ECA86 == 1
	//EF60 Temperaturen vom ECA86 gemessen
	if(buff[ECL_B_PTYPE] == ECL_P_TMP_ECA86 &&
	   buff[ECL_B_ADDR]  == ECL_A_ECA86_ECL300) {

		ECL_DEBUG_S("ECA86 Temp. ");

		if(buff[6] == ECL_P_TMP_ECA86_0) {
			ecl_data.tmp_ECA86[0] = (buff[3] << 9) | (buff[2] << 1);
			ecl_data.tmp_ECA86[1] = (buff[5] << 9) | (buff[4] << 1);

			ECL_DEBUG_S("1: ");
			ECL_DEBUG_I(ecl_data.tmp_ECA86[0] >> 8);
			ECL_DEBUG_C(' ');
			ECL_DEBUG_I(ecl_data.tmp_ECA86[1] >> 8);
			ECL_DEBUG_C('\n');
			return ECL_S_TMP_ECA86_1 | ECL_S_NORMAL;
		} else if(buff[6] == ECL_P_TMP_ECA86_1) {
			ecl_data.tmp_ECA86[2] = (buff[3] << 9) | (buff[2] << 1);
			ecl_data.tmp_ECA86[3] = (buff[5] << 9) | (buff[4] << 1);

			ECL_DEBUG_S("2: ");
			ECL_DEBUG_I(ecl_data.tmp_ECA86[2] >> 8);
			ECL_DEBUG_C(' ');
			ECL_DEBUG_I(ecl_data.tmp_ECA86[3] >> 8);
			ECL_DEBUG_C('\n');
			return ECL_S_TMP_ECA86_2 | ECL_S_NORMAL;
		}

		return ECL_S_NOT_HANDLED;
	}


	//FE62 Relais an ECA86 
	if(buff[ECL_B_PTYPE] == ECL_P_SET_RELAIS &&
	   buff[ECL_B_ADDR]  == ECL_A_ECL300_ECA86) {

		ECL_DEBUG_S("Relais: ");

	   	uint8_t msk_set = buff[6]; //Set-Maske
		uint8_t msk_res = buff[4]; //Reset-Maske

		ecl_data.relais = (0xFF ^ msk_res) & msk_set;
		
		ECL_DEBUG_H(ecl_data.relais);
		ECL_DEBUG_C('\n');

		return ECL_S_RELAIS | ECL_S_SET;
	}
#endif

#if ECL_USE_ECA60 == 1
	//AF04: Innentemperatur am ECA60 gemessen
	if(buff[ECL_B_PTYPE] == ECL_P_TMP_INDOOR && 
	   buff[ECL_B_ADDR]  == ECL_A_ECA60_ECL300) {
		
		ECL_DEBUG_S("Indoor Temp:");
		
		ecl_data.tmp_indoor = (buff[3] << 9) | (buff[2] << 1);

		ECL_DEBUG_I(ecl_data.tmp_indoor >> 8); //Wieder um 8 Stellen nach rechts schieben
		ECL_DEBUG_C('\n');
		return ECL_S_TMP_INDOOR | ECL_S_NORMAL;
	}
#endif

	//Auentemperatur
	//Die Temperatur wird hier als fixed-point-Variable gespeichert. Die oberen 8 Bit stellen die ganzzahlige Temperatur da.
	//Um die "echte" Temperatur zu erhalten, muss man also durch 256 teilen.
	if(buff[ECL_B_PTYPE] == ECL_P_TMP_OUTDOOR && 
	   buff[ECL_B_ADDR]  == ECL_A_ECL300_BCAST) {
		
		ECL_DEBUG_S("Outdoor Temp:");
		ecl_data.tmp_outdoor = (buff[3] << 9) | (buff[2] << 1);

		ECL_DEBUG_I(ecl_data.tmp_outdoor >> 8); //Wieder um 8 Stellen nach rechts schieben
		ECL_DEBUG_C('\n');

		/**
		 * Modi:
		 * 0: Reduzierte Temperatur
		 * 1: Optimierte Aufheizphase
		 * 2: Komforttemperatur
		 * 3: Optimierte Absenkphase
		 **/
		ecl_data.mode_water   = buff[5] >> 4;
		ecl_data.mode_heating = buff[5] & 0x0F;
		
		ECL_DEBUG_S("Water: ");
		switch(ecl_data.mode_water) {
			case 0: ECL_DEBUG_S("reduced"); break;
			case 1: ECL_DEBUG_S("optimized heating"); break;
			case 2: ECL_DEBUG_S("comfort"); break;
			case 3: ECL_DEBUG_S("optimized reduced"); break;
		}
		ECL_DEBUG_C('\n');

		ECL_DEBUG_S("Heating: ");
		switch(ecl_data.mode_heating) {
			case 0: ECL_DEBUG_S("reduced"); break;
			case 1: ECL_DEBUG_S("optimized heating"); break;
			case 2: ECL_DEBUG_S("comfort"); break;
			case 3: ECL_DEBUG_S("optimized reduced"); break;
		}
		ECL_DEBUG_C('\n');

		return ECL_S_TMP_OUTDOOR | ECL_S_NORMAL;
	}

	//F002, 0x11 Zeit (setzen)
	if((buff[ECL_B_PTYPE] == ECL_P_TIME &&
	   buff[ECL_B_ADDR]  == ECL_A_ECL300_BCAST)
#if ECL_USE_ECA60 == 1
	    ||
	   (buff[ECL_B_PTYPE] == ECL_P_SET_TIME &&
	   buff[ECL_B_ADDR]  == ECL_A_ECA60_ECL300)
#endif
	   ) {


#if ECL_DEBUG == 1
		if(buff[ECL_B_PTYPE] == ECL_P_SET_TIME) {
			ECL_DEBUG_S("Set ");
		}

		ECL_DEBUG_S("Time: ");
#endif
		ecl_data.time.second = buff[2];
		ecl_data.time.minute = buff[3];
		ecl_data.time.hour = buff[4];
		ecl_data.time.day = buff[5];
		ecl_data.time.wday = buff[7] >> 4;
		ecl_data.time.month = buff[7] & 0x0F;
		ecl_data.time.year = buff[6] + 1900;
		

		ECL_DEBUG_I(ecl_data.time.year);
		ECL_DEBUG_C('-');
		ECL_DEBUG_I(ecl_data.time.month);
		ECL_DEBUG_C('-');
		ECL_DEBUG_I(ecl_data.time.day);

		ECL_DEBUG_C(' ');
		ECL_DEBUG_I(ecl_data.time.hour);
		ECL_DEBUG_C(':');
		ECL_DEBUG_I(ecl_data.time.minute);
		ECL_DEBUG_C(':');
		ECL_DEBUG_I(ecl_data.time.second);
		ECL_DEBUG_C('\n');
#if ECL_USE_ECA60 == 1
		if(buff[ECL_B_PTYPE] == ECL_P_SET_TIME) {
			return ECL_S_TIME | ECL_S_SET;
		}
#endif
		return ECL_S_TIME | ECL_S_NORMAL;
	}

#if ECL_USE_ECA60 == 1
	//AF09 Abfrage Tagesprogramm
	if(buff[ECL_B_PTYPE] == ECL_P_REQ_PROG &&
	   buff[ECL_B_ADDR]  == ECL_A_ECA60_ECL300) {
		
		ecl_data.prog_day = buff[2] & 0x07;

		ECL_DEBUG_S("Daily Program for ");
		ECL_DEBUG_I(ecl_data.prog_day);
		ECL_DEBUG_S("?\n");

		return ECL_S_DAYPROGRAM | ECL_S_REQUEST;
	}

	//FA09 Rckgabe Tagesprogramm - nur auswertbar, wenn auch die Anfrage (Wochentag) dekodiert wurde
	if(buff[ECL_B_PTYPE] == ECL_P_REQ_PROG &&
	   buff[ECL_B_ADDR]  == ECL_A_ECL300_ECA60) {
		
		ECL_DEBUG_S("Daily Program: ");

		for(uint8_t i = 0; i < 6; i++) {
			ecl_data.prog[i] = buff[i+2];
			ECL_DEBUG_H(ecl_data.prog[i]);
		}

		ECL_DEBUG_C('\n');

		return ECL_S_DAYPROGRAM | ECL_S_NORMAL;
	}
#endif

	//AF05, FA05, FF05 Solltemperatur (nur brauchbar, wenn sie vom Regler gesendet wird)
	if(buff[ECL_B_PTYPE] == ECL_P_SET_MODE &&
	   (
#if ECL_USE_ECA60 == 1
	   buff[ECL_B_ADDR] == ECL_A_ECA60_ECL300 || 
#endif
	   buff[ECL_B_ADDR] == ECL_A_ECL300_ECA60 || buff[ECL_B_ADDR] == ECL_A_ECL300_ECL300)) {
		
		ECL_DEBUG_S("Mode: ");
		ecl_data.mode = buff[7] & 0x07;

#if ECL_DEBUG == 1
		switch(ecl_data.mode) {
			case 0: ECL_DEBUG_S("Manual"); break;
			case 1: ECL_DEBUG_S("Automatic"); break;
			case 2: ECL_DEBUG_S("permanent comfort"); break;
			case 3: ECL_DEBUG_S("permanent reduced"); break;
			case 4: ECL_DEBUG_S("standby"); break;
			default: ECL_DEBUG_S("unknown"); break;
		}
		ECL_DEBUG_C('\n');
#endif

		ecl_data.tmp_setpoint = (buff[3] >> 1) & 0x1F;

#if ECL_USE_ECA60 == 1
		//Bei den Temperaturen mssen - sofern die Abweichung negativ ist - die fhrenden Bits korrigiert werden.
		if(ecl_data.mode == ECL_DIFF_RELAX) {
			ecl_data.tmp_diff = buff[5] >> 1;
			if(ecl_data.tmp_diff & 0x20) {
				ecl_data.tmp_diff |= 0xC0;
			}
		} else if(ecl_data.mode == ECL_DIFF_AWAY) {
			ecl_data.tmp_diff = buff[6] >> 1;
			if(ecl_data.tmp_diff & 0x40) {
				ecl_data.tmp_diff |= 0x80;
			}
		} else {
			ecl_data.tmp_diff = 0;
		}
#endif

#if ECL_DEBUG == 1

		ECL_DEBUG_S("Setpoint: ");
		ECL_DEBUG_I(ecl_data.tmp_setpoint);

#if ECL_USE_ECA60 == 1
		switch(type) {
			case ECL_DIFF_RELAX: ECL_DEBUG_S("Relax"); break;
			case ECL_DIFF_AWAY:  ECL_DEBUG_S("Away");  break;
			default: break;
		}

		if(type == ECL_DIFF_RELAX || type == ECL_DIFF_AWAY) {
			ECL_DEBUG_I(ecl_data.tmp_diff);
		}

		ECL_DEBUG_C('\n');
#endif
#endif
		if(buff[ECL_B_ADDR] == ECL_A_ECA60_ECL300) {
			return ECL_S_SETPOINTMODE | ECL_S_SET;
		}
		return ECL_S_SETPOINTMODE | ECL_S_NORMAL;
	}

	return ECL_S_NOT_HANDLED;
}
