#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "m8help.h"
#include "ecl.h"

ISR(INT0_vect) {
	ecl_int();
}

ISR(TIMER0_OVF_vect) {
	ecl_tmr_ovf();
}

#define LED_PORT PORTD
#define LED_RD (1<<PD7)
#define LED_GN (1<<PD6)

#define LED_RED()   LED_PORT &= ~(LED_RD | LED_GN); LED_PORT |= LED_RD
#define LED_GREEN() LED_PORT &= ~(LED_RD | LED_GN); LED_PORT |= LED_GN
#define LED_BLACK() LED_PORT &= ~(LED_RD | LED_GN);
#define LED_ORANG() LED_PORT |= LED_RD | LED_GN

ISR(TIMER1_COMPA_vect) // timer_compare_a
{
	LED_BLACK();
}

int main() {

	uart_init();
	ecl_init();

	TCCR1B = (1<<WGM12)|(1<<CS11)|(1<<CS10);
	OCR1AH = 182;
	OCR1AL = 194;
	TIMSK |= (1<<OCIE1A);

	DDRD |= (1<<PD1) | LED_RD | LED_GN;
	DDRC |= (1<<PC5) | (1<<PC4) | (1<<PC3);

	sei();

	uart_putc(42);
//	uint8_t printnext = 0;

	while(1) {
	
		uint8_t state = ecl_decode();

		if(state == ECL_S_NOT_HANDLED) {
			LED_ORANG();
		} else if(state == ECL_S_NOT_READY || state == ECL_S_NOT_COMPLETE) {
			//LED_BLACK();
		} else if(state == ECL_S_CHECKSUM_ERROR) {
			LED_RED();
		} else {
			LED_GREEN();
		}
		
	}

}
